module Main (main) where

import Apriori (Itemset(..), Support, nextFreqItemsets, firstFreqItemsets,
 parNextFreqItemsets, parFirstFreqItemsets, monadParNextFreqItemsets, monadParFirstFreqItemsets)

import System.Exit (die)
import System.Environment (getArgs)
import qualified Data.List as List
import qualified Data.Set as Set


wordsWhen :: (Char -> Bool) -> String -> [String]
wordsWhen p s =
  case dropWhile p s of
    "" -> []
    s' -> w : wordsWhen p s''
        where (w, s'') = break p s'

getTransactions :: String -> [Itemset]
getTransactions content =
  map getItemset (lines content)
    where
      getItemset line = Itemset $ Set.fromList (wordsWhen (==',') line)

main :: IO()
main = do
  args <- getArgs
  case args of
    [file, supp] -> do
      let minSupp = read supp :: Support
      content <- readFile file
      let transactions = getTransactions content
      let freqItemsets = concat $ List.unfoldr (nextFreqItemsets minSupp transactions) (firstFreqItemsets minSupp transactions)
      print $ freqItemsets
    [file, supp, "parallel", n] -> do
      let minSupp = read supp :: Support
      let chunk = read n :: Int
      content <- readFile file
      let transactions = getTransactions content
      let freqItemsets = concat $ List.unfoldr (parNextFreqItemsets chunk minSupp transactions) (parFirstFreqItemsets chunk minSupp transactions)
      print $ freqItemsets
    [file, supp, "monad-par"] -> do
      let minSupp = read supp :: Support
      content <- readFile file
      let transactions = getTransactions content
      let freqItemsets = concat $ List.unfoldr (monadParNextFreqItemsets minSupp transactions) (monadParFirstFreqItemsets minSupp transactions)
      print $ freqItemsets
    _ -> do
      die "IllegalArgumentException: should be {filename, minSupport, ['parallel', listChunkSize] | ['monad-par']}"
